@@ -236,6 +236,7 @@ class Generator:
             msg = '%s GENERATOR: Starting generation for %s' % (self._mode.upper(), obj)
             debug.println(debug.LEVEL_INFO, msg, True)
 
+            debuginfo = lambda x: self._resultElementToString(x, False)
             assert(formatting)
             while True:
                 currentTime = time.time()
@@ -253,25 +254,38 @@ class Generator:
                         break
                     globalsDict[arg] = self._methodsDict[arg](obj, **args)
                     duration = "%.4f" % (time.time() - currentTime)
-                    debug.println(debug.LEVEL_ALL,
-                                  "           GENERATION TIME: %s  ---->  %s=%s" \
-                                  % (duration, arg, repr(globalsDict[arg])))
+                    if isinstance(globalsDict[arg], list):
+                        stringResult = " ".join(filter(lambda x: x, map(debuginfo, globalsDict[arg])))
+                        debug.println(debug.LEVEL_ALL,
+                                      "%sGENERATION TIME: %s  ---->  %s=[%s]" \
+                                      % (" " * 18, duration, arg, stringResult))
 
         except:
             debug.printException(debug.LEVEL_SEVERE)
             result = []
 
         duration = "%.4f" % (time.time() - startTime)
-        debug.println(debug.LEVEL_ALL, "           COMPLETION TIME: %s" % duration)
-        debug.println(debug.LEVEL_ALL, "%s GENERATOR: Results:" % self._mode.upper(), True)
-        for element in result:
-            debug.println(debug.LEVEL_ALL, "           %s" % element)
-
+        debug.println(debug.LEVEL_ALL, "%sCOMPLETION TIME: %s" % (' ' * 18, duration))
+        self._debugResultInfo(result)
         if args.get('isProgressBarUpdate') and result:
             self.setProgressBarUpdateTimeAndValue(obj)
 
         return result
 
+    def _resultElementToString(self, element, includeAll=True):
+        if not includeAll:
+            return str(element)
+
+        return "\n%s'%s'" % (" " * 18, element)
+
+    def _debugResultInfo(self, result):
+        if debug.LEVEL_ALL < debug.debugLevel:
+            return
+
+        info = "%s%s GENERATOR: Results: " % (" " * 18, self._mode.upper())
+        info += "%s" % " ".join(map(self._resultElementToString, result))
+        debug.println(debug.LEVEL_ALL, info)
+
     #####################################################################
     #                                                                   #
     # Name, role, and label information                                 #
@@ -287,6 +301,13 @@ class Generator:
         # Subclasses must override this.
         return []
 
+    def _fallBackOnDescriptionForName(self, obj, **args):
+        role = args.get('role', obj.getRole())
+        if role == pyatspi.ROLE_LABEL:
+            return False
+
+        return True
+
     def _generateName(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
         represent the name of the object.  If the object is directly
@@ -303,23 +324,10 @@ class Generator:
         """
         result = []
         self._script.pointOfReference['usedDescriptionForName'] = False
-        name = self._script.utilities.displayedText(obj)
-        if obj.getRole() == pyatspi.ROLE_COMBO_BOX:
-            children = self._script.utilities.selectedChildren(obj)
-            if not children and obj.childCount:
-                children = self._script.utilities.selectedChildren(obj[0])
-            children = children or [child for child in obj]
-            names = map(self._script.utilities.displayedText, children)
-            names = list(filter(lambda x: x, names))
-            if len(names) == 1:
-                name = names[0].strip()
-            elif len(children) == 1 and children[0].name:
-                name = children[0].name.strip()
-            elif not names and obj.name:
-                name = obj.name
+        name = obj.name
         if name:
             result.append(name)
-        else:
+        elif self._fallBackOnDescriptionForName(obj, **args):
             try:
                 description = obj.description
             except (LookupError, RuntimeError):
@@ -334,7 +342,7 @@ class Generator:
                 elif obj.parent and obj.parent.getRole() == pyatspi.ROLE_LINK:
                     link = obj.parent
                 if link:
-                    basename = self._script.utilities.linkBasename(link)
+                    basename = self._script.utilities.linkBasenameToName(link)
                     if basename:
                         result.append(basename)
         # To make the unlabeled icons in gnome-panel more accessible.
@@ -356,8 +364,16 @@ class Generator:
         the assumption being that the user was able to see the text prior
         to giving the widget focus.
         """
-        result = [x for x in obj.getAttributes() if x.startswith('placeholder-text:')]
-        return [x.replace('placeholder-text:', '') for x in result]
+        attrs = self._script.utilities.objectAttributes(obj)
+        placeholder = attrs.get('placeholder-text')
+        if placeholder and placeholder != obj.name:
+            return [placeholder]
+
+        placeholder = attrs.get('placeholder')
+        if placeholder and placeholder != obj.name:
+            return [placeholder]
+
+        return []
 
     def _generateLabelAndName(self, obj, **args):
         """Returns the label and the name as an array of strings for speech
@@ -367,10 +383,16 @@ class Generator:
         result = []
         label = self._generateLabel(obj, **args)
         name = self._generateName(obj, **args)
+        role = args.get('role', obj.role)
+        if not (label or name) and role == pyatspi.ROLE_TABLE_CELL:
+            descendant = self._script.utilities.realActiveDescendant(obj)
+            name = self._generateName(descendant)
+
         result.extend(label)
         if not len(label):
             result.extend(name)
-        elif len(name) and name[0].split() != label[0].split():
+        elif len(name) and name[0].split() != label[0].split() \
+             and not label[0].startswith(name[0]):
             result.extend(name)
         return result
 
@@ -379,11 +401,10 @@ class Generator:
         If the label cannot be found, the name will be used instead.
         If the name cannot be found, an empty array will be returned.
         """
-        result = []
-        result.extend(self._generateLabel(obj, **args))
+        result = self._generateLabel(obj, **args)
         if not result:
-            if obj.name and (len(obj.name)):
-                result.append(obj.name)
+            result = self._generateName(obj, **args)
+
         return result
 
     def _generateDescription(self, obj, **args):
@@ -403,17 +424,30 @@ class Generator:
         if role == pyatspi.ROLE_LABEL:
             return []
 
+        try:
+            name = obj.name
+            description = obj.description
+        except:
+            msg = "ERROR: Exception getting name and description for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            name = ""
+            description = ""
+
         if role == pyatspi.ROLE_ICON:
             name = self._script.utilities.displayedText(obj) or ""
-        else:
-            name = obj.name or ""
 
         result = []
-        if obj.description:
+        if description:
             label = self._script.utilities.displayedLabel(obj) or ""
-            desc = obj.description.lower()
+            desc = description.lower()
             if not (desc in name.lower() or desc in label.lower()):
                 result.append(obj.description)
+
+        if not result:
+            desc = self._script.utilities.displayedDescription(obj)
+            if desc:
+                result.append(desc)
+
         return result
 
     def _generateLabel(self, obj, **args):
@@ -428,6 +462,11 @@ class Generator:
             result.append(label)
         return result
 
+    def generateStatusBar(self, obj, **args):
+        """Returns an array of strings that represent a status bar."""
+
+        return self._generateStatusBar(obj, **args)
+
     #####################################################################
     #                                                                   #
     # Image information                                                 #
@@ -462,6 +501,18 @@ class Generator:
     def _generateHasLongDesc(self, obj, **args):
         return []
 
+    def _generateHasDetails(self, obj, **args):
+        return []
+
+    def _generateDetailsFor(self, obj, **args):
+        return []
+
+    def _generateAllDetails(self, obj, **args):
+        return []
+
+    def _generateHasPopup(self, obj, **args):
+        return []
+
     def _generateAvailability(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
         represent the grayed/sensitivity/availability state of the
@@ -528,7 +579,8 @@ class Generator:
         if not args.get('mode', None):
             args['mode'] = self._mode
         args['stringType'] = 'readonly'
-        if self._script.utilities.isReadOnlyTextArea(obj):
+        if obj.getState().contains(pyatspi.STATE_READ_ONLY) \
+           or self._script.utilities.isReadOnlyTextArea(obj):
             result.append(self._script.formatting.getString(**args))
         return result
 
@@ -634,6 +686,16 @@ class Generator:
             result.append(indicators[0])
         return result
 
+    def _generateCheckedStateIfCheckable(self, obj, **args):
+        if obj.getState().contains(pyatspi.STATE_CHECKABLE) \
+           or obj.getRole() == pyatspi.ROLE_CHECK_MENU_ITEM:
+            return self._generateCheckedState(obj, **args)
+
+        if obj.getState().contains(pyatspi.STATE_CHECKED):
+            return self._generateCheckedState(obj, **args)
+
+        return []
+
     def _generateMenuItemCheckedState(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
         represent the checked state of the menu item, only if it is
@@ -660,11 +722,13 @@ class Generator:
         args['stringType'] = 'expansion'
         indicators = self._script.formatting.getString(**args)
         state = obj.getState()
-        if state.contains(pyatspi.STATE_EXPANDABLE):
-            if state.contains(pyatspi.STATE_EXPANDED):
-                result.append(indicators[1])
-            else:
-                result.append(indicators[0])
+        if state.contains(pyatspi.STATE_COLLAPSED):
+            result.append(indicators[0])
+        elif state.contains(pyatspi.STATE_EXPANDED):
+            result.append(indicators[1])
+        elif state.contains(pyatspi.STATE_EXPANDABLE):
+            result.append(indicators[0])
+
         return result
 
     def _generateMultiselectableState(self, obj, **args):
@@ -694,6 +758,10 @@ class Generator:
         represent the row header for an object that is in a table, if
         it exists.  Otherwise, an empty array is returned.
         """
+
+        if args.get('readingRow'):
+            return []
+
         result = []
         header = self._script.utilities.rowHeaderForCell(obj)
         if not header:
@@ -740,6 +808,13 @@ class Generator:
         result.append(text)
         return result
 
+    def _generateSortOrder(self, obj, **args):
+        description = self._script.utilities.getSortOrderDescription(obj)
+        if not description:
+            return []
+
+        return [description]
+
     def _generateTableCell2ChildLabel(self, obj, **args):
         """Returns an array of strings for use by speech and braille for the
         label of a toggle in a table cell that has a special 2 child
@@ -844,12 +919,11 @@ class Generator:
         if self._script.utilities.isLayoutOnly(obj):
             return []
 
-        try:
-            table = obj.queryTable()
-        except:
+        rows, cols = self._script.utilities.rowAndColumnCount(obj)
+        if rows < 0 or cols < 0:
             return []
 
-        return [messages.tableSize(table.nRows, table.nColumns)]       
+        return [messages.tableSize(rows, cols)]
 
     def _generateTableCellRow(self, obj, **args):
         """Orca has a feature to automatically read an entire row of a table
@@ -860,62 +934,35 @@ class Generator:
         has changed.  Otherwise, it will return an array for just the
         current cell.
         """
+
+        presentAll = args.get('readingRow') == True \
+            or args.get('formatType') == 'detailedWhereAmI' \
+            or self._mode == 'braille' \
+            or self._script.utilities.shouldReadFullRow(obj)
+
+        if not presentAll:
+            return self._generateRealTableCell(obj, **args)
+
+        args['readingRow'] = True
         result = []
+        cells = self._script.utilities.getShowingCellsInSameRow(obj, forceFullRow=True)
 
-        try:
-            parentTable = obj.parent.queryTable()
-        except:
-            parentTable = None
-        isDetailedWhereAmI = args.get('formatType', None) == 'detailedWhereAmI'
-        readFullRow = self._script.utilities.shouldReadFullRow(obj)
-        if (readFullRow or isDetailedWhereAmI) and parentTable \
-           and (not self._script.utilities.isLayoutOnly(obj.parent)):
-            parent = obj.parent
-            index = self._script.utilities.cellIndex(obj)
-            row = parentTable.getRowAtIndex(index)
-            column = parentTable.getColumnAtIndex(index)
+        # Remove any pre-calcuated values which only apply to obj and not row cells.
+        doNotInclude = ['startOffset', 'endOffset', 'string']
+        otherCellArgs = args.copy()
+        for arg in doNotInclude:
+            otherCellArgs.pop(arg, None)
 
-            # This is an indication of whether we should speak all the
-            # table cells (the user has moved focus up or down a row),
-            # or just the current one (focus has moved left or right in
-            # the same row).
-            #
-            presentAll = True
-            if isDetailedWhereAmI:
-                if parentTable.nColumns <= 1:
-                    return result
-            elif "lastRow" in self._script.pointOfReference \
-               and "lastColumn" in self._script.pointOfReference:
-                pointOfReference = self._script.pointOfReference
-                presentAll = \
-                    (self._mode == 'braille') \
-                    or \
-                    ((pointOfReference["lastRow"] != row) \
-                     or ((row == 0 or row == parentTable.nRows-1) \
-                         and pointOfReference["lastColumn"] == column))
-            if presentAll:
-                args['readingRow'] = True
-                if self._script.utilities.isTableRow(obj):
-                    cells = [x for x in obj]
-                else:
-                    cells = [parentTable.getAccessibleAt(row, i) \
-                                 for i in range(parentTable.nColumns)]
-
-                for cell in cells:
-                    if not cell:
-                        continue
-                    state = cell.getState()
-                    showing = state.contains(pyatspi.STATE_SHOWING)
-                    if showing:
-                        cellResult = self._generateRealTableCell(cell, **args)
-                        if cellResult and result and self._mode == 'braille':
-                            result.append(braille.Region(
-                                object_properties.TABLE_CELL_DELIMITER_BRAILLE))
-                        result.extend(cellResult)
+        for cell in cells:
+            if cell == obj:
+                cellResult = self._generateRealTableCell(cell, **args)
             else:
-                result.extend(self._generateRealTableCell(obj, **args))
-        else:
-            result.extend(self._generateRealTableCell(obj, **args))
+                cellResult = self._generateRealTableCell(cell, **otherCellArgs)
+            if cellResult and result and self._mode == 'braille':
+                result.append(braille.Region(object_properties.TABLE_CELL_DELIMITER_BRAILLE))
+            result.extend(cellResult)
+
+        result.extend(self._generatePositionInList(obj, **args))
         return result
 
     #####################################################################
@@ -1010,6 +1057,15 @@ class Generator:
         attribute if it exists on the object.  [[[WDW - we should
         consider returning an empty array if there is no value.
         """
+
+        role = args.get('role', obj.getRole())
+        if role == pyatspi.ROLE_COMBO_BOX:
+            value = self._script.utilities.getComboBoxValue(obj)
+            return [value]
+
+        if role == pyatspi.ROLE_SEPARATOR and not obj.getState().contains(pyatspi.STATE_FOCUSED):
+            return []
+
         return [self._script.utilities.textForValue(obj)]
 
     #####################################################################
@@ -1095,7 +1151,7 @@ class Generator:
         if not (rad.getRole() == pyatspi.ROLE_TABLE_CELL and rad.childCount):
             return self._generateDisplayedText(rad, **args)
 
-        content = [self._script.utilities.displayedText(x).strip() for x in rad]
+        content = set([self._script.utilities.displayedText(x).strip() for x in rad])
         return [" ".join(filter(lambda x: x, content))]
 
     def _generateRealActiveDescendantRoleName(self, obj, **args ):
@@ -1126,6 +1182,9 @@ class Generator:
     def _generatePageSummary(self, obj, **args):
         return []
 
+    def _generatePositionInList(self, obj, **args):
+        return []
+
     def _generateProgressBarIndex(self, obj, **args):
         return []
 
@@ -1198,8 +1257,6 @@ class Generator:
             if self._script.utilities.isMathTableRow(obj):
                 return 'ROLE_MATH_TABLE_ROW'
         if self._script.utilities.isDPub(obj):
-            if self._script.utilities.isDPubFootnote(obj):
-                return 'ROLE_FOOTNOTE'
             if self._script.utilities.isLandmark(obj):
                 return 'ROLE_DPUB_LANDMARK'
             if obj.getRole() == pyatspi.ROLE_SECTION:
@@ -1210,10 +1267,26 @@ class Generator:
             return pyatspi.ROLE_STATIC
         if self._script.utilities.isBlockquote(obj):
             return pyatspi.ROLE_BLOCK_QUOTE
+        if self._script.utilities.isComment(obj):
+            return pyatspi.ROLE_COMMENT
+        if self._script.utilities.isContentDeletion(obj):
+            return 'ROLE_CONTENT_DELETION'
+        if self._script.utilities.isContentError(obj):
+            return 'ROLE_CONTENT_ERROR'
+        if self._script.utilities.isContentInsertion(obj):
+            return 'ROLE_CONTENT_INSERTION'
+        if self._script.utilities.isContentMarked(obj):
+            return 'ROLE_CONTENT_MARK'
+        if self._script.utilities.isContentSuggestion(obj):
+            return 'ROLE_CONTENT_SUGGESTION'
         if self._script.utilities.isLandmark(obj):
+            if self._script.utilities.isLandmarkRegion(obj):
+                return 'ROLE_REGION'
             return pyatspi.ROLE_LANDMARK
         if self._script.utilities.isFocusableLabel(obj):
             return pyatspi.ROLE_LIST_ITEM
+        if self._script.utilities.isDocument(obj) and 'Image' in pyatspi.listInterfaces(obj):
+            return pyatspi.ROLE_IMAGE
 
         return args.get('role', obj.getRole())
 
@@ -1249,6 +1322,9 @@ class Generator:
                 if isVertical:
                     return object_properties.ROLE_SPLITTER_HORIZONTAL
 
+        if self._script.utilities.isContentSuggestion(obj):
+            return object_properties.ROLE_CONTENT_SUGGESTION
+
         if self._script.utilities.isFeed(obj):
             return object_properties.ROLE_FEED
 
@@ -1326,14 +1402,14 @@ class Generator:
             else:
                 if self._script.utilities.isDPubCover(obj):
                     return object_properties.ROLE_COVER
-                if self._script.utilities.isDPubFootnote(obj):
-                    return object_properties.ROLE_FOOTNOTE
                 if self._script.utilities.isDPubPagebreak(obj):
                     return object_properties.ROLE_PAGEBREAK
                 if self._script.utilities.isDPubSubtitle(obj):
                     return object_properties.ROLE_SUBTITLE
 
         if self._script.utilities.isLandmark(obj):
+            if self._script.utilities.isLandmarkWithoutType(obj):
+                return ''
             if self._script.utilities.isLandmarkBanner(obj):
                 return object_properties.ROLE_LANDMARK_BANNER
             if self._script.utilities.isLandmarkComplementary(obj):
@@ -1350,6 +1426,8 @@ class Generator:
                 return object_properties.ROLE_LANDMARK_SEARCH
             if self._script.utilities.isLandmarkForm(obj):
                 role = pyatspi.ROLE_FORM
+        elif self._script.utilities.isComment(obj):
+            role = pyatspi.ROLE_COMMENT
 
         if not isinstance(role, (pyatspi.Role, Atspi.Role)):
             try:
